@@ -616,7 +616,6 @@ process_component_from_server (CalBackendWCAP *wcap, ECalComponent *comp)
 	ECalComponentDateTime dt_start, dt_end;
 	ECalComponentOrganizer organizer;
 	GSList *attendee_list, *l;
-	icaltimezone *izone = NULL;
 	
 	icalcomp = e_cal_component_get_icalcomponent (comp);
 
@@ -626,42 +625,23 @@ process_component_from_server (CalBackendWCAP *wcap, ECalComponent *comp)
 		icalcomponent_remove_property (icalcomp, prop);
 
 	/* Put everything in the right timezone */
-	prop = icalcomponent_get_first_property (icalcomp, ICAL_X_PROPERTY);
-	while (prop) {
-		const char *x_name, *x_val;
-
-		x_name = icalproperty_get_x_name (prop);
-		x_val = icalproperty_get_x (prop);
-
-		if (!strcmp (x_name, "X-NSCP-DTSTART-TZID")
-		    || !strcmp (x_name, "X-NSCP-DUE-TZID")) {
-			izone = cal_backend_wcap_get_timezone_from_tzid (wcap, x_val, TRUE);
-			break;			
-		}
-
-		prop = icalcomponent_get_next_property (icalcomp, ICAL_X_PROPERTY);
-	}
-
-	if (!izone)
-		izone = priv->server_default_zone;
-	
-	if (izone) {
+	if (priv->default_zone) {
 		ECalComponentDateTime dt;
 		icaltimezone *utc_zone;
 		icaltimezone *local_zone;
 
-		local_zone = icaltimezone_get_builtin_timezone (icaltimezone_get_location (izone));
+		local_zone = icaltimezone_get_builtin_timezone (icaltimezone_get_location (priv->default_zone));
 
 		utc_zone = icaltimezone_get_utc_timezone ();
 
 		e_cal_component_get_dtstart (comp, &dt);
 		if (dt.value && !icaltime_is_null_time (*dt.value)) {
-			icaltimezone_convert_time (dt.value, utc_zone, izone);
+			icaltimezone_convert_time (dt.value, utc_zone, priv->default_zone);
 			g_free ((char *)dt.tzid);
 			if (local_zone) {
 				dt.tzid = g_strdup (icaltimezone_get_tzid (local_zone));
 			} else  {
-				dt.tzid = g_strdup (icaltimezone_get_tzid (izone));
+				dt.tzid = g_strdup (icaltimezone_get_tzid (priv->default_zone));
 			}
 			e_cal_component_set_dtstart (comp, &dt);
 		}
@@ -669,12 +649,12 @@ process_component_from_server (CalBackendWCAP *wcap, ECalComponent *comp)
 		
 		e_cal_component_get_dtend (comp, &dt);
 		if (dt.value && !icaltime_is_null_time (*dt.value)) {
-			icaltimezone_convert_time (dt.value, utc_zone, izone);
+			icaltimezone_convert_time (dt.value, utc_zone, priv->default_zone);
 			g_free ((char *)dt.tzid);
 			if (local_zone) {
 				dt.tzid = g_strdup (icaltimezone_get_tzid (local_zone));
 			} else {
-				dt.tzid = g_strdup (icaltimezone_get_tzid (izone));
+				dt.tzid = g_strdup (icaltimezone_get_tzid (priv->default_zone));
 			}
 			e_cal_component_set_dtend (comp, &dt);
 		}
@@ -682,12 +662,12 @@ process_component_from_server (CalBackendWCAP *wcap, ECalComponent *comp)
 		
 		e_cal_component_get_due (comp, &dt);
 		if (dt.value && !icaltime_is_null_time (*dt.value)) {
-			icaltimezone_convert_time (dt.value, utc_zone, izone);
+			icaltimezone_convert_time (dt.value, utc_zone, priv->default_zone);
 			g_free ((char *)dt.tzid);
 			if (local_zone)
 				dt.tzid = g_strdup (icaltimezone_get_tzid (local_zone));
 			else 
-				dt.tzid = g_strdup (icaltimezone_get_tzid (izone));
+				dt.tzid = g_strdup (icaltimezone_get_tzid (priv->default_zone));
 			e_cal_component_set_due (comp, &dt);
 		}
 		e_cal_component_free_datetime (&dt);
@@ -704,8 +684,8 @@ process_component_from_server (CalBackendWCAP *wcap, ECalComponent *comp)
 			
 			ex = l->data;
 			/* convert from utc zone to local zone */
-			if (izone) {
-				icaltimezone_convert_time (ex->value, icaltimezone_get_utc_timezone(), izone);
+			if (priv->default_zone) {
+				icaltimezone_convert_time (ex->value, icaltimezone_get_utc_timezone(), priv->default_zone);
 				ex->value->is_utc = FALSE;
 			}
 
@@ -809,11 +789,44 @@ process_component_from_server (CalBackendWCAP *wcap, ECalComponent *comp)
 		e_cal_component_set_organizer (comp, NULL);
 	} 
 	
+	/* Alarms */
+	if (e_cal_component_has_alarms (comp)) {
+		ECalComponentDateTime dtstart;
+		icaltimezone *utc_zone = icaltimezone_get_utc_timezone ();
+		
+		e_cal_component_get_dtstart (comp, &dtstart);
+		if (dtstart.value) {
+			GList *auids, *l;
+			dtstart.value->is_date = 0;
+			icaltimezone_convert_time (dtstart.value, cal_backend_wcap_get_default_zone (wcap), utc_zone);
+			
+			auids = e_cal_component_get_alarm_uids (comp);
+			for (l = auids; l != NULL; l = l->next) {
+				const char *auid = l->data;
+				ECalComponentAlarm *alarm;
+				ECalComponentAlarmTrigger trigger;
+				
+				alarm = e_cal_component_get_alarm (comp, auid);
+				e_cal_component_alarm_get_trigger (alarm, &trigger);
+				if (trigger.type == E_CAL_COMPONENT_ALARM_TRIGGER_ABSOLUTE) {
+					struct icaldurationtype duration = icaltime_subtract (trigger.u.abs_time, *dtstart.value);
+
+					trigger.u.rel_duration = duration;
+					trigger.type = E_CAL_COMPONENT_ALARM_TRIGGER_RELATIVE_START;
+					e_cal_component_alarm_set_trigger (alarm, trigger);
+				}
+				e_cal_component_alarm_free (alarm);
+			}
+			cal_obj_uid_list_free (auids);
+		}
+		e_cal_component_free_datetime (&dtstart);
+	}
+
 	e_cal_component_abort_sequence (comp);
 }
 
 void
-cal_backend_wcap_add_component (CalBackendWCAP *wcap, icalcomponent *icalcomp, gboolean process, char **old_object)
+cal_backend_wcap_add_component (CalBackendWCAP *wcap, icalcomponent *icalcomp, gboolean process, char **old_object, char **new_object)
 {
 	CalBackendWCAPPrivate *priv = wcap->priv;
 	icalcomponent_kind kind;
@@ -863,13 +876,12 @@ cal_backend_wcap_add_component (CalBackendWCAP *wcap, icalcomponent *icalcomp, g
 	
 	/* Track the mangled uid components */
 	if (g_hash_table_lookup_extended (priv->objects, uid, &orig_key, &orig_value)) {
-		ECalComponentId id;
+		ECalComponentId *id;
 		char *calobj = e_cal_component_get_as_string (orig_value);
 
-		id.uid = g_strdup (uid);
-		id.rid = NULL;
-		e_cal_backend_notify_object_removed (E_CAL_BACKEND(wcap), &id, calobj, NULL);
-		g_free (id.uid);
+		id = e_cal_component_get_id (comp);
+		e_cal_backend_notify_object_removed (E_CAL_BACKEND(wcap), id, calobj, NULL);
+		e_cal_component_free_id (id);
 		g_free (calobj);
 		g_hash_table_remove (priv->objects, uid);
 		g_free (orig_key);
@@ -885,6 +897,8 @@ cal_backend_wcap_add_component (CalBackendWCAP *wcap, icalcomponent *icalcomp, g
 
 	char *calobj = e_cal_component_get_as_string (comp);
 	if(calobj) {
+		if (new_object && !*new_object)
+			*new_object = g_strdup (calobj);
 		e_cal_backend_notify_object_created (E_CAL_BACKEND (wcap), calobj);
 	}
 	g_free (calobj);
@@ -903,7 +917,7 @@ add_toplevel_component (CalBackendWCAP *wcap, icalcomponent *toplevel, gboolean
 			kind = icalcomponent_isa (subcomp);
 			if (kind == ICAL_VEVENT_COMPONENT ||
 			    kind == ICAL_VTODO_COMPONENT) {
-				cal_backend_wcap_add_component (wcap, subcomp, process, NULL);
+				cal_backend_wcap_add_component (wcap, subcomp, process, NULL, NULL);
 			} else if (kind == ICAL_VTIMEZONE_COMPONENT) {
 				add_timezone (wcap, subcomp);
 			}
@@ -911,7 +925,7 @@ add_toplevel_component (CalBackendWCAP *wcap, icalcomponent *toplevel, gboolean
 		}
 	} else if (kind == ICAL_VEVENT_COMPONENT ||
 		   kind == ICAL_VTODO_COMPONENT) {
-		cal_backend_wcap_add_component (wcap, toplevel, process, NULL);
+		cal_backend_wcap_add_component (wcap, toplevel, process, NULL, NULL);
 	} else if (kind == ICAL_VTIMEZONE_COMPONENT) {
 		add_timezone (wcap, toplevel);
 	}
@@ -965,16 +979,15 @@ cal_backend_wcap_remove_component (CalBackendWCAP *wcap, const char *uid, CalObj
 		
 		e_cal_component_get_uid (remove_comp, &mangled_uid);
 		if (g_hash_table_lookup_extended (priv->objects, mangled_uid, &orig_key, &orig_value)) {
-			ECalComponentId id;
+			ECalComponentId *id;
 			char *calobj = e_cal_component_get_as_string (remove_comp);
 
 			if (old_object && !*old_object)
 				*old_object = g_strdup (calobj);
-			id.uid = g_strdup (mangled_uid);
-			id.rid = NULL;
-			e_cal_backend_notify_object_removed (E_CAL_BACKEND(wcap), &id, calobj, NULL);
-			g_free (id.uid);
 			g_hash_table_remove (priv->objects, mangled_uid);
+			id = e_cal_component_get_id (remove_comp);
+			e_cal_backend_notify_object_removed (E_CAL_BACKEND(wcap), id, calobj, NULL);
+			e_cal_component_free_id (id);
 			g_free (calobj);
 			g_free (orig_key);
 			g_object_unref (G_OBJECT (orig_value));
